//
//  CoreDataModel.m
//  ToDo-RestKit
//
//  Created by Rodger Higgins on 4/9/13.
//  Copyright (c) 2013 Rodger Higgins. All rights reserved.
//

#import "CoreDataModel.h"
#import "NSNotificationCenter+MainThread.h"

#define kCoreDataReadyName              @"CoreDataReady"

@interface CoreDataModel ()

@property (nonatomic, strong) NSManagedObjectModel *managedObjectModel;

@end

@implementation CoreDataModel
@synthesize mainContext = _mainContext;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;
@synthesize managedObjectModel = _managedObjectModel;


+ (CoreDataModel *)sharedDataModel {
    static CoreDataModel *_sharedDataModel = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedDataModel = [[CoreDataModel alloc] init];
    });
    
    return _sharedDataModel;
}

#pragma mark - Abstract methods for filename information

- (NSString *)modelName
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

- (NSString *)pathToModel
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

- (NSString *)storeFilename
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

- (NSString *)pathToLocalStore
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

- (NSDictionary *)persistentStoreOptions
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

- (NSString *)storeType
{
    [NSException raise:NSInternalInconsistencyException
                format:@"You must override %@ in a subclass", NSStringFromSelector(_cmd)];
    
    return nil;
}

#pragma mark - Property methods

- (NSManagedObjectContext *)mainContext
{
    if ( _mainContext == nil ) {
        _mainContext = [[NSManagedObjectContext alloc] init];
        _mainContext.persistentStoreCoordinator = [self persistentStoreCoordinator];
    }
    
    return _mainContext;
}

- (NSManagedObjectModel *)managedObjectModel
{
    if ( _managedObjectModel == nil ) {
        NSURL *storeURL = [NSURL fileURLWithPath:[self pathToModel]];
        _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:storeURL];
    }
    
    return _managedObjectModel;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if ( _persistentStoreCoordinator == nil ) {
        DLog(@"Store Path: %@", [self pathToLocalStore]);
        NSURL *storeURL = [NSURL fileURLWithPath:[self pathToLocalStore]];
        NSPersistentStoreCoordinator *psc = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
        NSDictionary *options = [self persistentStoreOptions];
        
        NSError *e = nil;
        if (![psc addPersistentStoreWithType:NSSQLiteStoreType
                               configuration:nil
                                         URL:storeURL
                                     options:options
                                       error:&e]) {
            
            ALog(@"Unresolved error: %@ %@", e, [e userInfo]);
        }
        
        _persistentStoreCoordinator = psc;
    }
    
    return _persistentStoreCoordinator;
}


#pragma mark -
#pragma mark Launch Method

- (void)startCoreData
{
    dispatch_queue_t queue = NULL;
    queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    dispatch_async(queue, ^{
        // Initialize everything
        [self mainContext];
        
        // Send notification
        [self sendDataModelOnline];

    });
}


#pragma mark -
#pragma mark NSNotification methods

- (void)sendDataModelOnline
{
    [[NSNotificationCenter defaultCenter] postNotificationOnMainThreadName:kCoreDataReadyName object:self];
}


- (void)registerDataModelOnline:(id)object selector:(SEL)method object:(id)obj
{
    [[NSNotificationCenter defaultCenter]
                    addObserver:object
                       selector:method
                           name:kCoreDataReadyName
                         object:obj];
}

- (void)unregisterDataModelOnline:(id)object
{
    [[NSNotificationCenter defaultCenter] removeObserver:object];
}

@end
